#include "clarke_balanced.h"
#include <stdio.h>

void clarke_balanced_test(void);

float arr_Ialpha[NUM_SAMPLES] @ "RAM_data_region";	// Balanced Clarke transform output Ialpha
float arr_Ibeta[NUM_SAMPLES] @ "RAM_data_region";	// Balanced Clarke transform output Ibeta
float error_tolerance = 0.001f;						// Balanced Clarke transform output error tolerance
int accuracy_test;									// Balanced Clarke transform output accuracy test
float max_error = 0.0f;								// Balanced Clarke transform output maximum error
unsigned int t_ncycles;

int main(int argc, char *argv[])
{
	init_platform();

	clarke_balanced_test();

	return 0;
}

/**************************************************************************************************************************
	Function name	:	clarke_balanced_test
	Description		:	This function tests Clarke (balanced) transformation
	Arguments		:	None.
	Return value	:	None.
**************************************************************************************************************************/
void clarke_balanced_test(void)
{
	int i;
	float * pIa = (float *)std_Ia;	// Pointer to test vector Ia
	float * pIb = (float *)std_Ib;	// Pointer to test vector Ib

	unsigned int t_overhead, t_cn;
	volatile cycle_count_t t_start, t_stop;

	get_cycle_count(&t_start);
	get_cycle_count(&t_stop);
	t_overhead = subtract_cycle_count(&t_start, &t_stop);	// Measure cycle counting overhead to be subtracted

	// Zero Ialpha and Ibeta
	for( i = 0 ; i < (NUM_SAMPLES) ; i++ )
	{
		arr_Ialpha[i] = 0.0f;
		arr_Ibeta[i] = 0.0f;
	}

	// Perform Balanced Clarke Transform NUM_SAMPLES times and measure cycle count
	get_cycle_count(&t_start);
	for( i = 0 ; i < (NUM_SAMPLES) ; i++ )
	{
		clarke_balanced_xform(pIa[i], pIb[i], &arr_Ialpha[i], &arr_Ibeta[i]);
	}
	get_cycle_count(&t_stop);
	t_cn = (subtract_cycle_count(&t_start, &t_stop)) - t_overhead; // Subtract cycle counting overhead

#if TEST_MODE==M4_CMSISLIB
	printf("Clarke Transform (balanced) on Cortex-M4 using CMSIS library ");
#else // TEST_MODE==M4_CMSISLIB
	printf("Clarke Transform (balanced) on Cortex-M4 ");
#endif // TEST_MODE==M4_CMSISLIB

	printf("\nIalpha Accuracy Test ");
	// Verify accuracy of test output Ialpha with standard output
	accuracy_test = verify_accuracy_arrays(arr_Ialpha, (float *)std_Ialpha, NUM_SAMPLES, error_tolerance, &max_error);
	if (accuracy_test)
		printf("Successful ");
	else
		printf("Failure; Max error = %f ", max_error);

	printf("\nIbeta Accuracy Test ");
	// Verify accuracy of test output Ibeta with standard output
	accuracy_test = verify_accuracy_arrays(arr_Ibeta, (float *)std_Ibeta, NUM_SAMPLES, error_tolerance, &max_error);
	if (accuracy_test)
		printf("Successful ");
	else
		printf("Failure; Max error = %f ", max_error);

	printf("\nNumber of Clarke (balanced) Computations = %d; Total number of cycles = %d\n", NUM_SAMPLES, t_cn);

	t_ncycles = t_cn;
	return;
}
