/*************************************************************************************************************************
									File Details
**************************************************************************************************************************
	File name							:	CGUInit.h
	Purpose								:	Includes the macros, variables and function declarations for CGUInit.c
	Author								:	Nabeel
	VDSP version used for testing		:	..
											Version:
											Build id:
	Hardware used						:	N/A
	Connection details					:	N/A
	Guidelines for change				:	N/A
**************************************************************************************************************************/
#ifndef CGUINIT_H_
#define CGUINIT_H_

#include "device.h"
#include <stdio.h>
#include <stdarg.h>
#include <stdint.h>


#define CLKIN_CM40x     30000000	/* CLKIN to processor in Hz */
#define CLKIN_CM41x     30000000	/* CLKIN to processor in Hz */

// ADSP_CM40x Clock Max Limits
#define PLLCLK_MAX_CM40x      800
#define CCLK_MAX_CM40x        250
#define SYSCLK_MAX_CM40x      100
#define DCLK_MAX_CM40x        60
#define OUTCLK_MAX_CM40x      50

// ADSP_CM41x Clock Max Limits
#define PLLCLK_MAX_CM41x      1020
#define CCLK_MAX_CM41x        240
#define SYSCLK_MAX_CM41x      100
#define S0CLK_MAX_CM41x       100
#define DCLK_MAX_CM41x        100
#define OUTCLK_MAX_CM41x      450

/*  PLL Multiplier and Divisor Selections (Required Value, Bit Position) */
#define MSEL(X)   		((X << BITP_CGU_CTL_MSEL) 				& BITM_CGU_CTL_MSEL)			/* PLL Multiplier Select */
#define DF(X)	  		((X << BITP_CGU_CTL_DF) 				& BITM_CGU_CTL_DF )				/* Divide frequency[true or false] */
#define CSEL(X)   		((X  << BITP_CGU_DIV_CSEL)				& BITM_CGU_DIV_CSEL)			/* Core Clock Divisor Select */
#define SYSSEL(X) 		((X  << BITP_CGU_DIV_SYSSEL)  			& BITM_CGU_DIV_SYSSEL)			/* System Clock Divisor Select */
#define S0SEL(X)  		((X  << BITP_CGU_DIV_S0SEL)				& BITM_CGU_DIV_S0SEL)			/* SCLK0 Divisor Select  */
#define S1SEL(X)  		((X  << BITP_CGU_DIV_S1SEL)				& BITM_CGU_DIV_S1SEL)			/* SCLK1 Divisor Select  */
#define DSEL(X)   		((X  << BITP_CGU_DIV_DSEL)				& BITM_CGU_DIV_DSEL)			/* DDR Clock Divisor Select */
#define OSEL(X)   		((X  << BITP_CGU_DIV_OSEL)				& BITM_CGU_DIV_OSEL)			/* OUTCLK Divisor Select */
#define CLKOUTSEL(X)    ((X  << BITP_CGU_CLKOUTSEL_CLKOUTSEL)	& BITM_CGU_CLKOUTSEL_CLKOUTSEL)	/* CLKOUT select	*/
#define USBCLKSEL(X)    ((X  << BITP_CGU_CLKOUTSEL_USBCLKSEL)	& BITM_CGU_CLKOUTSEL_USBCLKSEL)	/* USBCLKSEL select */


#define DIV_MASK		(BITM_CGU_DIV_CSEL | BITM_CGU_DIV_S0SEL | BITM_CGU_DIV_SYSSEL |BITM_CGU_DIV_S1SEL | BITM_CGU_DIV_DSEL | BITM_CGU_DIV_OSEL)	/*  Mask for dividers in CGU_DIV register  */
#define CTL_MASK		(BITM_CGU_CTL_MSEL | BITM_CGU_CTL_DF)	/* Mask for MSEL and DF in CGU_CTL register	*/


#define BITM_ROM_SYSCTRL_CGU_DIV		0x0001
#define BITM_ROM_SYSCTRL_CGU_AUTO_DIS	0x0002
#define BITM_ROM_SYSCTRL_CGU_CTL		0x0004

#define BITM_CDU_STAT_CLKO8		0x00000100

typedef struct CGU_Settings	/* Structure to pass the values to CGU driver */
{
	uint32_t ctl_MSEL:7;
	uint32_t ctl_DF:1;
	uint32_t div_CSEL:5;
	uint32_t div_SYSSEL:5;
	uint32_t div_S0SEL:3;
	uint32_t div_S1SEL:3;
	uint32_t div_DSEL:5;
	uint32_t div_OSEL:7;
	uint32_t reserved0:28;	/* 64 bits */
}CGU_Settings;


__ramfunc void CGU_Init(int iMultiplier, \
              int bDivideFrequency, \
              int iCCLKSel, \
              int iDDCLKSel, \
              int iSCLK0Sel, \
              int iSCLK1Sel, \
              int iSysCLKSel, \
              int iOutClock, \
              int iClkOut);

__ramfunc uint32_t CGU_Init_All(uint32_t uiCguNo, uint32_t uiClkinsel, CGU_Settings *pClocks,  uint32_t uiClkoutsel);

__ramfunc uint32_t CGU0_write(uint32_t dActionFlags, CGU_Settings *pCGU_Settings, uint32_t uiClkOutSel);
__ramfunc uint32_t CGU1_write(uint32_t dActionFlags, CGU_Settings *pCGU_Settings, uint32_t uiClkOutSel);
__ramfunc void Program_Cgu0(uint32_t dActionFlags, uint32_t dPllBypassed, uint32_t dPllEnabled, uint32_t dNewCguDiv, CGU_Settings *pCGU_Settings);
__ramfunc void Program_Cgu1(uint32_t dActionFlags, uint32_t dPllBypassed, uint32_t dPllEnabled, uint32_t dNewCguDiv, CGU_Settings *pCGU_Settings);

__ramfunc void CGU_Freq_Check_Round(CGU_Settings *pClocks, float uiClkinValue);


__ramfunc void Active_To_Fullon(unsigned int uiCGU_ID);

__ramfunc void ActivePLLDis_To_Active(unsigned int uiCGU_ID);

#ifdef ADI_ADSP_CM41X_M4 
__ramfunc void CM41xFlash_SetPerformanceMode(unsigned int core_clock);
#endif /* ADI_ADSP_CM41X_M4 */

#endif /* CGUINIT_H_ */
