#ifndef __COMMON_H__
#define __COMMON_H__

#include "startup.h"
#include "system.h"
#include <math.h>
#include "CGUInit.h"

//Choose CCLK and SysCLK settings (in MHz)
#define CCLK_SCLK_240_96
//#define CCLK_SCLK_240_80
//#define CCLK_SCLK_225_100
//#define CCLK_SCLK_200_100
//#define CCLK_SCLK_150_100
//#define CCLK_SCLK_75_75

//#define DEBUG		// Print additional Debug statements

#define M4_MATHLIB 				0
#define M4_CMSISLIB				1
#define MATH_ACCELERATOR		2
#define MATH_M4_CODEINTERLEAVE	3

#define TEST_MODE M4_MATHLIB	//Choose one from above

/* Do Not Modify below this */

#if TEST_MODE==M4_CMSISLIB
	#include <arm_math.h>	// Include CMSIS library
#endif /* TEST_MODE==M4_CMSISLIB */


#if (TEST_MODE==MATH_ACCELERATOR) || (TEST_MODE==MATH_M4_CODEINTERLEAVE)

	#include "adi_math.h"	// Include Math Unit library

	/** Start calculation of cosine **/
	#pragma inline=forced
	inline void cosf_write  ( float32_t x ) { pADI_MATH0->COSF   = x; return; }

	/** Read cosine result **/
	#pragma inline=forced
	inline float32_t cosf_read  ( ) { return pADI_MATH0->RES1; }

	/** Start calculation of sine **/
	#pragma inline=forced
	inline void sinf_write  ( float32_t x ) { pADI_MATH0->SINF   = x; return; }

	/** Read sine result **/
	#pragma inline=forced
	inline float32_t sinf_read  ( ) { return pADI_MATH0->RES1; }

#endif /* (TEST_MODE==MATH_ACCELERATOR) || (TEST_MODE==MATH_M4_CODEINTERLEAVE) */

typedef const unsigned int const_u32_t;

extern volatile unsigned int SysTickCnt;	/* SysTick Counter */

typedef struct
{
   unsigned int tick;
   unsigned int count;
} cycle_count_t;

#pragma inline=forced
static void get_cycle_count(volatile cycle_count_t * x)
{
   x->tick = SysTickCnt;
   x->count = SysTick->VAL;
}

void Local_SysTick_Config(void);
static unsigned long long cycleCountToULL(cycle_count_t * x);
unsigned int subtract_cycle_count(volatile cycle_count_t * _start, volatile cycle_count_t * _end);
void init_platform(void);
int verify_accuracy(float Result, float Expected, float Tolerance, float *pError);
int verify_accuracy_arrays(const float *pResult, const float *pExpected, int N,  float Tolerance, float *pMaxError);

#endif /* __COMMON_H__ */
