#include "ipark.h"
#include <stdio.h>

void ipark_test(void);

float arr_Ialpha[NUM_SAMPLES] @ "RAM_data_region";	// Inverse Park transform output Ialpha
float arr_Ibeta[NUM_SAMPLES] @ "RAM_data_region";	// Inverse Park transform output Ibeta
// Inverse Park transform output error tolerance
#if TEST_MODE==M4_CMSISLIB
	float error_tolerance = 0.006f;					// CMSIS accuracy is low because of radian to degree conversion
#else // TEST_MODE
	float error_tolerance = 0.001f;
#endif // TEST_MODE
int accuracy_test;									// Inverse Park transform output accuracy test
float max_error = 0.0f;								// Inverse Park transform output maximum error
unsigned int t_ncycles;

int main(int argc, char *argv[])
{
	init_platform();

	ipark_test();

	return 0;
}

/**************************************************************************************************************************
	Function name	:	ipark_test
	Description		:	This function tests Inverse Park transformation
	Arguments		:	None.
	Return value	:	None.
**************************************************************************************************************************/
void ipark_test(void)
{
	int i;
	float * p_Id = (float *)std_Id;			// Pointer to test vector Id
	float * p_Iq = (float *)std_Iq;			// Pointer to test vector Iq
	float * p_theta = (float *)std_theta;	// Pointer to test vector theta

	unsigned int t_overhead, t_cn;
	volatile cycle_count_t t_start, t_stop;

	get_cycle_count(&t_start);
	get_cycle_count(&t_stop);
	t_overhead = subtract_cycle_count(&t_start, &t_stop);	// Measure cycle counting overhead to be subtracted

	// Zero Ialpha and Ibeta
	for( i = 0 ; i < (NUM_SAMPLES) ; i++ )
	{
		arr_Ialpha[i] = 0.0f;
		arr_Ibeta[i] = 0.0f;
	}

	// Perform Inverse Park Transform NUM_SAMPLES times and measure cycle count
#if TEST_MODE==MATH_M4_CODEINTERLEAVE
	get_cycle_count(&t_start);
	ipark_xform_interl(p_Id, p_Iq, p_theta, arr_Ialpha, arr_Ibeta, NUM_SAMPLES);
	get_cycle_count(&t_stop);
#else // TEST_MODE==MATH_M4_CODEINTERLEAVE
	get_cycle_count(&t_start);
	for( i = 0 ; i < (NUM_SAMPLES) ; i++ )
	{
		ipark_xform(p_Id[i], p_Iq[i], p_theta[i], &arr_Ialpha[i], &arr_Ibeta[i]);
	}
	get_cycle_count(&t_stop);
#endif // TEST_MODE==MATH_M4_CODEINTERLEAVE
	t_cn = (subtract_cycle_count(&t_start, &t_stop)) - t_overhead; // Subtract cycle counting overhead

#if TEST_MODE==M4_MATHLIB
	printf("Inverse Park Transform on Cortex-M4 using math.h library       ");
#elif TEST_MODE==M4_CMSISLIB
	printf("Inverse Park Transform on Cortex-M4 using CMSIS library     ");
#elif TEST_MODE==MATH_ACCELERATOR
	printf("Inverse Park Transform on Math Unit ");
#elif TEST_MODE==MATH_M4_CODEINTERLEAVE
	printf("Inverse Park Transform with Math Unit and Cortex-M4 interleaved ");
#endif // TEST_MODE

	printf("\nIalpha Accuracy Test ");
	// Verify accuracy of test output Ialpha with standard output
	accuracy_test = verify_accuracy_arrays(arr_Ialpha, (float *)std_Ialpha, NUM_SAMPLES, error_tolerance, &max_error);
	if (accuracy_test)
		printf("Successful ");
	else
		printf("Failure; Max error = %f ", max_error);

	printf("\nIbeta Accuracy Test ");
	// Verify accuracy of test output Ibeta with standard output
	accuracy_test = verify_accuracy_arrays(arr_Ibeta, (float *)std_Ibeta, NUM_SAMPLES, error_tolerance, &max_error);
	if (accuracy_test)
		printf("Successful ");
	else
		printf("Failure; Max error = %f ", max_error);

	printf("\nNumber of Inverse Park Computations = %d; Total number of cycles = %d\n", NUM_SAMPLES, t_cn);	

	t_ncycles = t_cn;
	return;
}
