#include "park.h"
#include <stdio.h>

void park_test(void);

float arr_Id[NUM_SAMPLES] @ "RAM_data_region";	// Park transform output Id
float arr_Iq[NUM_SAMPLES] @ "RAM_data_region";	// Park transform output Iq
// Park transform output error tolerance
#if TEST_MODE==M4_CMSISLIB
	float error_tolerance = 0.005f;				// CMSIS accuracy is low because of radian to degree conversion
#elif (TEST_MODE==MATH_ACCELERATOR) || (TEST_MODE==MATH_M4_CODEINTERLEAVE)
	float error_tolerance = 0.001f;
#elif TEST_MODE==M4_MATHLIB
	float error_tolerance = 0.002f;				// math.h accuracy is lower than Math Unit
#endif // TEST_MODE
int accuracy_test;								// Park transform output accuracy test
float max_error = 0.0f;							// Park transform output maximum error
unsigned int t_ncycles;

int main(int argc, char *argv[])
{
	init_platform();

	park_test();

	return 0;
}

/**************************************************************************************************************************
	Function name	:	park_test
	Description		:	This function tests Park transformation
	Arguments		:	None.
	Return value	:	None.
**************************************************************************************************************************/
void park_test(void)
{
	int i;
	float * p_Ialpha = (float *)std_Ialpha;	// Pointer to test vector Ialpha
	float * p_Ibeta = (float *)std_Ibeta;	// Pointer to test vector Ibeta
	float * p_theta = (float *)std_theta;	// Pointer to test vector theta

	unsigned int t_overhead, t_cn;
	volatile cycle_count_t t_start, t_stop;

	get_cycle_count(&t_start);
	get_cycle_count(&t_stop);
	t_overhead = subtract_cycle_count(&t_start, &t_stop);	// Measure cycle counting overhead to be subtracted

	// Zero Id and Iq
	for( i = 0 ; i < (NUM_SAMPLES) ; i++ )
	{
		arr_Id[i] = 0.0f;
		arr_Iq[i] = 0.0f;
	}

	// Perform Park Transform NUM_SAMPLES times and measure cycle count
#if TEST_MODE==MATH_M4_CODEINTERLEAVE
	get_cycle_count(&t_start);
	park_xform_interl(p_Ialpha, p_Ibeta, p_theta, arr_Id, arr_Iq, NUM_SAMPLES);
	get_cycle_count(&t_stop);
#else // TEST_MODE==MATH_M4_CODEINTERLEAVE
	get_cycle_count(&t_start);
	for( i = 0 ; i < (NUM_SAMPLES) ; i++ )
	{
		park_xform(p_Ialpha[i], p_Ibeta[i], p_theta[i], &arr_Id[i], &arr_Iq[i]);
	}
	get_cycle_count(&t_stop);
#endif // TEST_MODE==MATH_M4_CODEINTERLEAVE
	t_cn = (subtract_cycle_count(&t_start, &t_stop)) - t_overhead; // Subtract cycle counting overhead

#if TEST_MODE==M4_MATHLIB
	printf("Park Transform on Cortex-M4 using math.h library            ");
#elif TEST_MODE==M4_CMSISLIB
	printf("Park Transform on Cortex-M4 using CMSIS library");
#elif TEST_MODE==MATH_ACCELERATOR
	printf("Park Transform on Math Unit ");
#elif TEST_MODE==MATH_M4_CODEINTERLEAVE
	printf("Park Transform with Math Unit and Cortex-M4 interleaved ");
#endif // TEST_MODE

	printf("\nId Accuracy Test ");
	// Verify accuracy of test output Id with standard output
	accuracy_test = verify_accuracy_arrays(arr_Id, (float *)std_Id, NUM_SAMPLES, error_tolerance, &max_error);        
	if (accuracy_test)
		printf("Successful ");
	else
		printf("Failure; Max error = %f ", max_error);

	printf("\nIq Accuracy Test ");
	// Verify accuracy of test output Iq with standard output
	accuracy_test = verify_accuracy_arrays(arr_Iq, (float *)std_Iq, NUM_SAMPLES, error_tolerance, &max_error);
	if (accuracy_test)
		printf("Successful ");
	else
		printf("Failure; Max error = %f ", max_error);

	printf("\nNumber of Park Computations = %d; Total number of cycles = %d\n", NUM_SAMPLES, t_cn);

	t_ncycles = t_cn;
	return;
}
